
from modeltranslation.translator import translator

from attributes.models import Attribute, AttributeOption


translator.register(Attribute, fields=['name'])
translator.register(AttributeOption, fields=['name'])



# Generated by Django 2.0.9 on 2020-03-12 17:31

from django.db import migrations, models
import django.db.models.deletion
from attributes.settings import ATTRIBUTES_CATEGORY_MODEL, ATTRIBUTES_ENTRY_MODEL


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attribute',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(db_index=True, editable=False, verbose_name='order')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', models.CharField(blank=True, db_index=True, max_length=255, verbose_name='Code')),
                ('type', models.PositiveSmallIntegerField(choices=[(1, 'Text'), (2, 'Integer'), (3, 'True / False'), (5, 'Options')], default=1, verbose_name='Type')),
                ('is_required', models.BooleanField(default=False, help_text='You will not be able to update record without filling this field', verbose_name='Required')),
                ('is_visible', models.BooleanField(default=True, help_text='Display this attribute for users', verbose_name='Is visible')),
                ('is_filter', models.BooleanField(default=False, help_text='Display this attribute in records filter', verbose_name='Is filter')),
                ('categories', models.ManyToManyField(blank=True, related_name='attributes', to=ATTRIBUTES_CATEGORY_MODEL, verbose_name='Categories')),
            ],
            options={
                'verbose_name': 'Attribute',
                'verbose_name_plural': 'Attributes',
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='AttributeOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('attr', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='options', to='attributes.Attribute', verbose_name='Attribute')),
            ],
            options={
                'verbose_name': 'Attribute option',
                'verbose_name_plural': 'Attribute options',
            },
        ),
        migrations.CreateModel(
            name='AttributeValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value_text', models.TextField(blank=True, null=True)),
                ('value_int', models.IntegerField(blank=True, null=True)),
                ('value_bool', models.NullBooleanField()),
                ('attr', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='values', to='attributes.Attribute')),
                ('entry', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attr_values', to=ATTRIBUTES_ENTRY_MODEL)),
                ('value_option', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='attr_values', to='attributes.AttributeOption')),
            ],
            options={
                'ordering': ('attr__order',),
            },
        ),
        migrations.AlterUniqueTogether(
            name='attributevalue',
            unique_together={('attr', 'entry')},
        ),
        migrations.AlterUniqueTogether(
            name='attributeoption',
            unique_together={('attr', 'name')},
        ),
    ]




from django import forms
from django.utils.translation import ugettext
from django.core.exceptions import ValidationError
from django.utils.functional import cached_property

from apps.products.models import Product

from attributes.models import (
    Attribute,
    AttributeValue,
    AttributeOption)


class FilterForm(forms.Form):

    def __init__(self, category, *args, **kwargs):

        self._attributes = Attribute\
            .objects\
            .visible()\
            .for_filter()\
            .for_categories([category])

        super().__init__(*args, **kwargs)

        for attr in self._attributes:
            self.fields[attr.full_slug] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple,
                label=attr.name,
                required=False)

    def set_options(self, entries):

        if not entries:
            self.fields = {}
            return

        choices = {attr.id: [] for attr in self._attributes}

        attr_values = AttributeValue.objects.filter(
            attr__in=self._attributes,
            entry__in=entries
        ).values_list('id', flat=True)

        options = AttributeOption\
            .objects\
            .filter(attr_values__in=attr_values)\
            .order_by('name')\
            .distinct()

        for option in options:
            choices[option.attr_id].append((option.id, option, ))

        for attr in self._attributes:
            if choices[attr.id]:
                self.fields[attr.full_slug].choices = choices[attr.id]
            else:
                del self.fields[attr.full_slug]

    def get_value_ids(self):

        ids = []

        for attr in self._attributes:
            ids += self.data.getlist(attr.full_slug)

        return ids

    def _get_available_options(self):

        added_options = []

        options = {attr.pk: [] for attr in self._attributes}

        attr_values = AttributeValue.objects.filter(
            attribute__in=self._attributes,
            entry__in=self._entries
        ).select_related('value_option')

        for value in attr_values:

            option = value.value_option

            if option not in added_options:
                added_options.append(option)
                options[value.attribute_id].append(option)

        return options


class AttributesForm(forms.ModelForm):

    def __init__(
            self,
            data=None,
            files=None,
            instance=None,
            initial=None,
            **kwargs):

        if instance.pk:
            initial = self._get_initial_data(instance)

        super().__init__(
            data=data,
            files=files,
            instance=instance,
            initial=initial,
            **kwargs)

        for attr in self._attributes:

            fields = attr.build_form_fields()

            self.fields.update(fields)

    def _get_initial_data(self, instance):

        initial = {}

        values = {
            v.attr.full_slug: v.get_value()
            for v in AttributeValue.objects.filter(
                attr__in=self._attributes,
                entry=instance
            )
        }

        for attr in self._attributes:
            initial[attr.full_slug] = values.get(attr.full_slug)

        return initial

    def clean(self):

        data = self.cleaned_data

        for attr in self._attributes:

            if attr.has_options:
                new_option = data.get(attr.get_option_form_field_name())

                if new_option:
                    option, c = attr.options.get_or_create(name=new_option)
                    data[attr.full_slug] = option

                if not data.get(attr.full_slug) and attr.is_required:
                    raise ValidationError({
                        attr.full_slug: ugettext('{} is required').format(
                            attr.name)
                    })

        return data

    def commit(self, instance):

        for attr in Attribute.objects.for_categories([instance.category]):

            if attr.full_slug in self.cleaned_data:
                value = self.cleaned_data[attr.full_slug]
                attr.save_value(self.instance, value)

        return instance

    @cached_property
    def _attributes(self):
        return list(Attribute.objects.all())

    class Media:
        js = ('attrs/form.js', )

    class Meta:
        model = Product
        fields = ['id']




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class AttributesAppConfig(AppConfig):
    name = 'attributes'
    verbose_name = _('Attributes')

default_app_config = 'attributes.AttributesAppConfig'




from django.forms import Field, ValidationError
from django.utils.translation import ugettext_lazy as _

from attributes.widgets import AttributesFormFieldWidget
from attributes.forms import AttributesForm


class AttributesFormField(Field):

    widget = AttributesFormFieldWidget
    form = None

    def init_form(self, *args, **kwargs):

        self.form = AttributesForm(*args, **kwargs)
        self.widget.form = self.form

    def clean(self, *args, **kwargs):

        if not self.form.is_valid():
            raise ValidationError(_('Form is invalid.'))

        return self.form.cleaned_data

    def commit(self, *args, **kwargs):
        return self.form.commit(*args, **kwargs)




from importlib import import_module

from django.apps import apps
from django import forms
from django.contrib import admin
from django.conf import settings

from slugify import slugify_url

from attributes.models import Attribute, AttributeOption


def _get_attr_option_inline_base_class():

    if apps.is_installed('modeltranslation'):
        return import_module('modeltranslation.admin').TranslationTabularInline

    return admin.TabularInline


class AttributeOptionInline(_get_attr_option_inline_base_class()):

    model = AttributeOption
    extra = 0


class AttributeForm(forms.ModelForm):

    def clean_slug(self):

        data = self.cleaned_data

        if data.get('slug'):
            return data['slug']

        if apps.is_installed('modeltranslation'):
            name = data.get('name_{}'.format(settings.LANGUAGE_CODE))
        else:
            name = data.get('name')

        return slugify_url(name, separator='_')

    class Meta:
        model = Attribute
        fields = [
            'categories',
            'name',
            'slug',
            'type',
            'is_required',
            'is_visible',
            'is_filter'
        ]




from django.conf import settings

from attributes import defaults


ATTRIBUTES_CATEGORY_MODEL = getattr(
    settings, 'ATTRIBUTES_CATEGORY_MODEL', defaults.ATTRIBUTES_CATEGORY_MODEL)

ATTRIBUTES_ENTRY_MODEL = getattr(
    settings, 'ATTRIBUTES_ENTRY_MODEL', defaults.ATTRIBUTES_ENTRY_MODEL)
